/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software; you can redistribute it and/or modify it under the terms of
the GNU General Public License, version 2, as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

/// Handles all events from SDL_PollEvent.
module mde.events;

import mde.Init;

import mde.scheduler;
import global = mde.global;

import mde.input.input;
import mde.input.exception;

import derelict.sdl.events;

import tango.util.log.Log : Log, Logger;

private Logger logger;
static this() {
    logger = Log.getLogger ("mde.events");

    Init.addFunc (&initInput);
}

// An Init function
void initInput () {
    try {
        global.input = new Input();
        global.input.loadConfig ();         // (may also create instance)
                
        Scheduler.perFrame (&pollEvents);
    } catch (Exception e) {
        init.setFailure ();                 // must clean up properly
    }
}

void pollEvents (double) {
    SDL_Event event;
    while (SDL_PollEvent (&event)) {
        switch (event.type) {
            case SDL_QUIT:
                logger.info ("Quit requested");
                global.run = false;
                break;
            default:
                try {
                    global.input (event);
                } catch (InputClassException e) {
                    logger.warn ("Caught input exception; event will be ignored. Exception was:");
                    logger.warn (e.msg);
                }
        }
    }
}
